/*
 * TismTool © Tis Veugen, 2010-2011
 * Simulation, Verification
 *
 * TsmClass.h
 * version 3
 */
#ifndef TSM_CLASS_H
#define TSM_CLASS_H

#include <sstream>
#include "TsmBase.h"
#include "TsmFsm.h"
#include "TsmTrigger.h"
using namespace std;

namespace TsmRTE
{
  class TsmProxyUser;
  class TsmInterfacePort;

  class TsmClass : public TsmBase
  {
  public:
    static const int  NativePortId = -1;
    int        mNrFsms;
    TsmFsm**   oFsms;
    bool       IsMock;
    string     SnapState;
    string     SnapCall;
    int        SeqId;

    TsmClass()
      : mNrFsms( 0 )
      , IsMock( false )
      , SnapState( "" )
      , SnapCall( "" )
    {
    }

    virtual ~TsmClass()
    {
      if ( mNrFsms > 0 )
      {
        for ( int i=0; i<mNrFsms; i++ )
        {
          delete oFsms[i];
        }
        delete [] oFsms;
      }
    }

    virtual void SetProxyUser( TsmProxyUser* oProxyUser )
    {
      // ignore
    }

    virtual TsmInterfacePort* GetProvidedInterface( string sPortName )
    {
      stringstream sb;
      sb << "9500: GetProvidedInterface should be overriden for '" << sPortName << "' @ '" << this->FullName();
      throw TsmException( sb.str() );
    }

    virtual void SetRequiredInterface( string sPortName, TsmInterfacePort* oProvidedInterface )
    {
      stringstream sb;
      sb << "9510: SetRequiredInterface should be overriden for '" << sPortName << "' @ '" << this->FullName();
      throw TsmException( sb.str() );
    }

    virtual void ExecuteTrigger( TsmTrigger* oTrigger )
    {
    }

    virtual void ExecuteNative( int iCmd )
    {
      TsmTrigger lTrigger( iCmd );
      lTrigger.PortId = NativePortId;
      ExecuteTrigger( &lTrigger );
    }

    virtual void TsmInitialise( void* oObject )
    {
    }

    virtual void TsmConnect( void* oObject )
    {
    }

    virtual void TsmPrepare( void* oObject )
    {
    }

    virtual void TsmRun( void* oObject )
    {
    }

    virtual void TsmHalt( void* oObject )
    {
    }

    virtual void TsmCease( void* oObject )
    {
    }

    virtual void TsmDisconnect( void* oObject )
    {
    }

    virtual void TsmTerminate( void* oObject )
    {
    }

    void TsmMention( void* oObject );
    void TsmReset1();
    void TsmReset2();
    string TsmSnap();
    void TsmCheckVisit();

    virtual bool IsTsmProxy()
    {
      return false;
    }

    virtual bool IsTsmProxyUser()
    {
      return false;
    }
  };
}
#endif
